package fr.itsonus.bousoleplussbackend.usecase;

import fr.itsonus.bousoleplussbackend.domain.auth.exception.AlreadyExistingUserException;
import fr.itsonus.bousoleplussbackend.domain.auth.model.User;
import fr.itsonus.bousoleplussbackend.domain.auth.spi.UserCacheRepository;
import fr.itsonus.bousoleplussbackend.domain.notification.NotificationService;
import fr.itsonus.bousoleplussbackend.domain.quiz.spi.QuestionCacheRepository;
import jakarta.transaction.Transactional;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

@Service
public class UserCreationUseCase {

    private static final String DEFAULT_BUNDLE_LABEL = "Boussole standard";
    private final UserCacheRepository userRepository;
    private final PasswordEncoder passwordEncoder;
    private final BundleUseCase bundleUseCase;
    private final QuestionCacheRepository questionCacheRepository;
    private final NotificationService notificationService;

    private final String mailFrom;
    private final String frontEndUrl;


    UserCreationUseCase(@Value("${spring.mail.from}") String mailFrom,
                        @Value("${frontend.url}") String frontEndUrl,
                        UserCacheRepository userRepository, PasswordEncoder passwordEncoder,
                        BundleUseCase bundleUseCase, QuestionCacheRepository questionCacheRepository,
                        NotificationService notificationService) {
        this.userRepository = userRepository;
        this.passwordEncoder = passwordEncoder;
        this.bundleUseCase = bundleUseCase;
        this.questionCacheRepository = questionCacheRepository;
        this.mailFrom = mailFrom;
        this.frontEndUrl = frontEndUrl;
        this.notificationService = notificationService;
    }

    @Transactional
    public void createUserAndDefaultBundle(final User user) {
        checkUserExistence(user);
        var newUser = userRepository.save(user.toBuilder().password(passwordEncoder.encode(user.getPassword())).build());
        bundleUseCase.create(DEFAULT_BUNDLE_LABEL, questionCacheRepository.findDefaultQuestions(), newUser);

        notificationService.notify(mailFrom, user.getEmail(), "Merci d’avoir créé votre compte !",
                String.format(
                        """
                        Votre compte a bien été créé sur la Boussole PLUSS.

                        Vous pouvez vous connecter avec votre identifiant et le mot de passe choisi lors de l'inscription sur %s et commencer votre évaluation.
                        """, frontEndUrl));
    }

    private void checkUserExistence(User user) {
        if (userRepository.existsByEmail(user.getEmail())) {
            throw new AlreadyExistingUserException("User already registered with the e-mail provided");
        }
    }
}
